@@ -129,10 +129,56 @@ char GetAttributeType(XPtr<Attribute> attribute) {
 
 // [[Rcpp::export]]
 NumericVector GetAttributeDimensions(XPtr<Attribute> attribute) {
-  DataSpace dataspace = attribute->getSpace();
-  int ndim = dataspace.getSimpleExtentNdims();
-  vector<hsize_t> dims_out(ndim);
-  dataspace.getSimpleExtentDims(&dims_out[0], NULL);
-  return NumericVector(dims_out.begin(), dims_out.end());
+	try {
+	  DataSpace dataspace = attribute->getSpace();
+	  int ndim = dataspace.getSimpleExtentNdims();
+
+	  NumericVector out;
+	  if(ndim > 0) {
+		vector<hsize_t> dims_out(ndim);
+		dataspace.getSimpleExtentDims(&dims_out[0], NULL);
+		out = NumericVector(dims_out.begin(), dims_out.end());
+	  } else { // Assume scalar Attribute
+		out = NumericVector(1);
+		out[0] = 1;
+	  }
+	  return out;
+	} catch (Exception& error) {
+		 string msg = error.getDetailMsg() + " in " + error.getFuncName();
+		 throw Rcpp::exception(msg.c_str());
+	}
 }
 
+// [[Rcpp::export]]
+CharacterVector GetAttributeNames_CommonFG(XPtr<CommonFG> file) {
+	try {
+		CharacterVector(out);
+		H5Aiterate2(file->getLocId(), H5_INDEX_NAME, H5_ITER_INC, NULL, attr_info, &out);
+		return out;
+	} catch (Exception& error) {
+		 string msg = error.getDetailMsg() + " in " + error.getFuncName();
+		 throw Rcpp::exception(msg.c_str());
+	}
+}
+
+// [[Rcpp::export]]
+CharacterVector GetAttributeNames_DataSet(XPtr<DataSet> file) {
+	try {
+		CharacterVector(out);
+		H5Aiterate2(file->getId(), H5_INDEX_NAME, H5_ITER_INC, NULL, attr_info, &out);
+		return out;
+	} catch (Exception& error) {
+		 string msg = error.getDetailMsg() + " in " + error.getFuncName();
+		 throw Rcpp::exception(msg.c_str());
+	}
+}
+
+herr_t attr_info(hid_t loc_id, const char *name, const H5A_info_t *ainfo, void *opdata) {
+//attr_info(hid_t loc_id, const char *name, void *opdata) {
+	try {
+		((CharacterVector *) opdata)->push_back(name);
+		return 0;
+	 } catch (Exception& error) {
+		 return 1;
+	 }
+}
